package applicationRepository;

import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Timestamp;
import java.util.Collections;
import java.util.Date;
import java.util.Vector;

import dataTypes.ResourceType;
import dataTypes.Statistics;

public class ApplicationHistoryDatabase {

	private Connection connection = null;

	private String url = null;

	public ApplicationHistoryDatabase(String url) {
		this.url = url;

		try {
			Class.forName("org.h2.Driver");
		} catch (ClassNotFoundException e) {
			e.printStackTrace();
		}
		createTable("ApplicationHistory");
	}

	public ApplicationHistoryDatabase() {
		this("jdbc:h2:file:../database/ApplicationHistoryDatabase");
	}

	public synchronized double getMean(String applicationName, ResourceType resource) {
		String columnName = getColumnName(resource);
		double mean = 0;

		String sql = null;
		PreparedStatement statement = null;

		sql = "SELECT AVG(" + columnName
				+ ") FROM ApplicationHistory WHERE APP_NAME = ?";
		try {
			statement = connection().prepareStatement(sql);
			statement.setString(1, applicationName);
			ResultSet result = statement.executeQuery();

			if (result.next()) {
				mean = result.getDouble(1);
			}
		} catch (SQLException e) {
			e.printStackTrace();
		}

		return mean;
	}

	private String getColumnName(ResourceType resource) {
		String name = "";
		if (resource == ResourceType.executionTime) {
			name = "EXEC_TIME";
		} else if (resource == ResourceType.memory) {
			name = "MEMORY";
		}
		return name;
	}

	// TODO remove me
	private synchronized void printDebug(String applicationName, String attribute) {
		String sql = null;
		PreparedStatement statement = null;

		sql = "SELECT * FROM ApplicationHistory WHERE APP_NAME = ?";
		try {
			statement = connection().prepareStatement(sql);
			statement.setString(1, applicationName);
			ResultSet result = statement.executeQuery();

			while (result.next()) {
				System.err.println(result.getLong(attribute));
			}

			statement.close();
			close();

		} catch (SQLException e) {
			e.printStackTrace();
		}

	}

	public synchronized double getStandardDeviation(String applicationName,
			ResourceType resource) {
		String columnName = getColumnName(resource);
		double stdDev = 0;

		String sql = null;
		PreparedStatement statement = null;

		sql = "SELECT STDDEV_SAMP(" + columnName
				+ ") FROM ApplicationHistory WHERE APP_NAME = ?";
		try {
			statement = connection().prepareStatement(sql);
			statement.setString(1, applicationName);
			ResultSet result = statement.executeQuery();

			if (result.next()) {
				stdDev = result.getDouble(1);
			}
		} catch (SQLException e) {
			e.printStackTrace();
		}

		return stdDev;
	}

	public synchronized double getMedian(String applicationName, ResourceType resource) {
		String columnName = getColumnName(resource);
		double median = 0;
		String sql = null;
		PreparedStatement statement = null;
		Vector<Double> list = new Vector<Double>(size(applicationName));

		sql = "SELECT * FROM ApplicationHistory WHERE APP_NAME = ?";
		try {
			statement = connection().prepareStatement(sql);
			statement.setString(1, applicationName);
			ResultSet result = statement.executeQuery();

			while (result.next()) {
				list.add(result.getDouble(columnName));
			}

			Collections.sort(list);
			int medianIndex = list.size() / 2;
			if (list.size() > 0) {
				median = list.elementAt(medianIndex);
			}

			statement.close();
			close();

		} catch (SQLException e) {
			e.printStackTrace();
		}

		return median;
	}

	public synchronized int size(String applicationName) {
		int size = 0;
		String sql = null;
		PreparedStatement statement = null;

		sql = "SELECT * FROM ApplicationHistory WHERE APP_NAME = ?";
		try {
			statement = connection().prepareStatement(sql);
			statement.setString(1, applicationName);
			ResultSet result = statement.executeQuery();

			result.last();
			size = result.getRow();

			statement.close();
			close();

		} catch (SQLException e) {
			e.printStackTrace();
		}

		return size;
	}

	public synchronized void addExecution(String applicationName,
			double executionTime, long memory, double bogoMips) {
		System.err.println("Db::addExecution(" + applicationName + ", "
				+ executionTime + ", " + memory + ", " + bogoMips + ")");
		String sql = null;
		PreparedStatement statement = null;
		Timestamp timestamp = new Timestamp(new Date().getTime());

		sql = "INSERT INTO ApplicationHistory VALUES (?, ?, ?, ?, ?)";
		try {
			statement = connection().prepareStatement(sql);
			statement.setTimestamp(1, timestamp);
			statement.setString(2, applicationName);
			statement.setDouble(3, executionTime);
			statement.setDouble(4, memory);
			statement.setDouble(5, bogoMips);

			statement.execute();
			statement.close();

			close();
		} catch (SQLException e) {
			e.printStackTrace();
		}

	}

	private synchronized void createTable(String tableName) {
		String sql = null;
		PreparedStatement statement = null;

		System.err.println("ApplicationHistoryDatabase created.");

		sql = "CREATE TABLE IF NOT EXISTS "
				+ tableName
				+ "(ID TIMESTAMP PRIMARY KEY, APP_NAME VARCHAR(255), EXEC_TIME DOUBLE, MEMORY DOUBLE, MIPS DOUBLE) ;";
		try {
			statement = connection().prepareStatement(sql);
			statement.execute();
			statement.close();
			close();
		} catch (SQLException e) {
			e.printStackTrace();
		}
	}

	private synchronized void dropTable(String tableName) {
		String sql = null;
		PreparedStatement statement = null;

		System.err.println("ApplicationHistoryDatabase dropped.");

		sql = "DROP TABLE IF EXISTS " + tableName + " ;";
		try {
			statement = connection().prepareStatement(sql);
			statement.execute();
			statement.close();
			close();
		} catch (SQLException e) {
			e.printStackTrace();
		}
	}

	private synchronized Connection connection() {
		if (this.connection == null) {
			try {
				this.connection = DriverManager.getConnection(this.url);
				this.connection.setAutoCommit(true);
			} catch (SQLException e) {
				e.printStackTrace();
			}
		}
		return this.connection;
	}

	private synchronized void close() {
		try {
			connection().close();
			this.connection = null;
		} catch (SQLException e) {
			e.printStackTrace();
		}
	}

	public void purge() {
		String sql = null;
		PreparedStatement statement = null;

		sql = "DROP TABLE ApplicationHistory ;";
		try {
			statement = connection().prepareStatement(sql);
			statement.execute();
			statement.close();
			close();
		} catch (SQLException e) {
			e.printStackTrace();
		}
	}

	public double getStatistic(String applicationName, ResourceType resource,
			Statistics statistic) {
		double value = 0.0;
		if (statistic == Statistics.mean) {
			value = getMean(applicationName, resource);
		} else if (statistic == Statistics.median) {
			value = getMedian(applicationName, resource);
		} else if (statistic == Statistics.standardDeviation) {
			value = getStandardDeviation(applicationName, resource);
		}
		return value;
	}
}
